#ifndef __TDB_H__
#define __TDB_H__

/*
   Unix SMB/CIFS implementation.
   Samba database functions
   Copyright (C) Andrew Tridgell 1999

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 * ??-???-????	Jean-Yves Collot?	SAMBA 2.2.8
 * 19-Sep-2004	J. Malmberg		Update for VMS 8.2
 */

#ifndef _INCLUDES_H
#include <signal.h>
typedef volatile sig_atomic_t SIG_ATOMIC_T;
#endif

/* flags to tdb_store() */
#define TDB_REPLACE 1
#define TDB_INSERT 2
#define TDB_MODIFY 3

/* flags for tdb_open() */
#define TDB_DEFAULT 0 /* just a readability place holder */
#define TDB_CLEAR_IF_FIRST 1
#define TDB_INTERNAL 2 /* don't store on disk */
#define TDB_NOLOCK   4 /* don't do any locking */
#define TDB_NOMMAP   8 /* don't use mmap */
#define TDB_CONVERT 16 /* convert endian (internal use) */
#define TDB_BIGENDIAN 32 /* header is big-endian (internal use) */

#define TDB_ERRCODE(code, ret) ((tdb->ecode = (code)), ret)

/* error codes */
enum TDB_ERROR {TDB_SUCCESS=0, TDB_ERR_CORRUPT, TDB_ERR_IO, TDB_ERR_LOCK,
                TDB_ERR_OOM, TDB_ERR_EXISTS, TDB_ERR_NOEXIST, TDB_ERR_NOLOCK, TDB_ERR_LOCK_TIMEOUT };

#ifndef u32
#define u32 unsigned
#endif

typedef struct {
	char *dptr;
	size_t dsize;
} TDB_DATA;

typedef u32 tdb_len;
typedef u32 tdb_off;

/* this is the context structure that is returned from a db open */
struct tdb_record {
	unsigned int hash_key;
	int key_size;
	int data_size;
	char value[2036];
};
/*
 * Use hack to keep isam structure private to vms_tdb_isam.
 */
#ifdef __DECC
#pragma message save
#pragma message disable dollarid
#endif
typedef struct tdb_isam_context {
#ifdef RMS$_NORMAL
    struct FAB fab;			/* file access block */
    struct RAB rab;			/* record access block */
    struct XABKEY key;			/* key attributes for primary key */
    char key_name[32];			/* From prolog 3 key descriptor */
    unsigned short last_deleted_rfa[3];   /* Caches RFA of 'deleted' rec. */
#endif
    int rec_valid;			/* True if tdb_record+last_hash valid */
    int key_version;			/* hash algorithm for primary key */
    u32 last_hash;
    u32 key_mask;
    u32 last_deleted_hash;
    short last_deleted_valid;
    struct tdb_record tdb_record;	/* Buffer for I/O operations */
} TDB_ISAM;

#ifdef __DECC
#pragma message restore
#endif


typedef struct tdb_context {
	char *name; /* the name of the database */
       	struct tdb_method_set *fn;
	TDB_ISAM *isam;
	int read_only; /* opened read-only */
	enum TDB_ERROR ecode; /* error code for last tdb error */
	u32 flags; /* the flags passed to tdb_open */
/*	struct tdb_traverse_lock travlocks; */ /* current traversal locks */
	struct tdb_context *next; /* all tdbs to avoid multiple opens */
	void (*log_fn)(struct tdb_context *tdb, int level, const char *, ...); /* logging function */
	int open_flags; /* flags used in the open - needed by reopen */
	TDB_DATA tdb_dbuf;		/* points to last record returned */
	TDB_DATA tdb_dkey;		/* Key of last record */

	int hash_mask;			/* hash table size is mask+1 */
	int hash_position;		/* context for first/next_key */
	struct tdb_int_data *cur_int_data;
	struct tdb_int_data *hash_table[1];	/* variable allocation */
} TDB_CONTEXT;

struct tdb_int_data {
    struct tdb_int_data *next_hash;	/* next in hash chain */
    TDB_DATA data;			/* Handle to data (may change) */
    size_t key_size;
    char key[1];			/* variably-size allocation */
};

typedef int (*tdb_traverse_func)(TDB_CONTEXT *, TDB_DATA, TDB_DATA, void *);
typedef void (*tdb_log_func)(TDB_CONTEXT *, int , const char *, ...);

typedef struct tdb_method_set {
    char *name;				/* Name of the method set */
    TDB_DATA (*fetch)(TDB_CONTEXT *, TDB_DATA );
    int (*delete)(TDB_CONTEXT *, TDB_DATA );
    int (*store) (TDB_CONTEXT *, TDB_DATA, TDB_DATA, int );
    int (*close) ( TDB_CONTEXT * );
    TDB_DATA (*firstkey) ( TDB_CONTEXT * );
    TDB_DATA (*nextkey) ( TDB_CONTEXT *, TDB_DATA );
    int (*traverse) ( TDB_CONTEXT *, tdb_traverse_func fn, void *state );
    int (*exists) ( TDB_CONTEXT *, TDB_DATA );
    int (*lockkeys) ( TDB_CONTEXT *, u32, TDB_DATA * );
    void (*unlockkeys) ( TDB_CONTEXT * );
    int (*lockall) ( TDB_CONTEXT * );
    void (*unlockall) ( TDB_CONTEXT * );

    int (*chainlock) ( TDB_CONTEXT *, TDB_DATA );
    int (*chainunlock) ( TDB_CONTEXT *, TDB_DATA );

    void (*dump_all) ( TDB_CONTEXT * );
    int (*printfreelist) ( TDB_CONTEXT * );
} TDB_METHODS;

TDB_CONTEXT *tdb_open(const char *name, int hash_size, int tdb_flags,
		      int open_flags, mode_t mode);
TDB_CONTEXT *tdb_open_ex(const char *name, int hash_size, int tdb_flags,
			 int open_flags, mode_t mode,
			 tdb_log_func log_fn);
int tdb_open_isam ( TDB_CONTEXT *tdb, const char *name, int hash_size,
	mode_t mode );

int tdb_reopen(TDB_CONTEXT *tdb);
int tdb_reopen_all(void);
void tdb_logging_function(TDB_CONTEXT *tdb, tdb_log_func);
enum TDB_ERROR tdb_error(TDB_CONTEXT *tdb);
const char *tdb_errorstr(TDB_CONTEXT *tdb);
TDB_DATA tdb_fetch(TDB_CONTEXT *tdb, TDB_DATA key);
TDB_DATA tdb_dupe(TDB_DATA src );
int tdb_delete(TDB_CONTEXT *tdb, TDB_DATA key);
int tdb_store(TDB_CONTEXT *tdb, TDB_DATA key, TDB_DATA dbuf, int flag);
int tdb_close(TDB_CONTEXT *tdb);
TDB_DATA tdb_firstkey(TDB_CONTEXT *tdb);
TDB_DATA tdb_nextkey(TDB_CONTEXT *tdb, TDB_DATA key);
int tdb_traverse(TDB_CONTEXT *tdb, tdb_traverse_func fn, void *state);
int tdb_exists(TDB_CONTEXT *tdb, TDB_DATA key);
int tdb_lockkeys(TDB_CONTEXT *tdb, u32 number, TDB_DATA keys[]);
void tdb_unlockkeys(TDB_CONTEXT *tdb);
int tdb_lockall(TDB_CONTEXT *tdb);
void tdb_unlockall(TDB_CONTEXT *tdb);

/* Low level locking functions: use with care */
void tdb_set_lock_alarm(SIG_ATOMIC_T *palarm);
/* void tdb_set_lock_alarm(sig_atomic_t *palarm); */
int tdb_chainlock(TDB_CONTEXT *tdb, TDB_DATA key);
int tdb_chainunlock(TDB_CONTEXT *tdb, TDB_DATA key);

/* Debug functions. Not used in production. */
void tdb_dump_all(TDB_CONTEXT *tdb);
int tdb_printfreelist(TDB_CONTEXT *tdb);

extern TDB_DATA tdb_null;

#endif /* tdb.h */
